﻿using System.Collections.ObjectModel;
using System.Data.Services.Client;
using ChinookWinApp.ChinookService;
using ChinookWinApp.Common;
using System;
using System.Linq;
using Windows.UI.Popups;
using System.Net;

namespace ChinookWinApp
{
    public class MainViewModel : BindableBase
    {
        private const string BaseUrl = @"http://dulpltsbar07/ChinookWCFDataService/ChinooDataService.svc/";
        private ChinookEntities _chinookEntities = new ChinookEntities(new Uri(BaseUrl));
        private Cookie _cookie;

        public MainViewModel()
        {
            ArtistLetterSelectedCommand = new SimpleCommand<ArtistAlphabetInfo>(OnArtistLetterSelected);
            AddArtistCommand = new SimpleCommand<object>(OnAddArtist);
            DeleteArtistCommand = new SimpleCommand<object>(OnDeleteArtist);
            SaveArtistCommand = new SimpleCommand<object>(OnSaveArtists);
            EditArtistCommand = new SimpleCommand<object>(OnEditeArtist);
            _chinookEntities.SendingRequest += OnSendingRequest;
            Load();
        }

        void OnSendingRequest(object sender, SendingRequestEventArgs e)
        {
            e.RequestHeaders["Cookie"] = _cookie.ToString();
        }

        private bool _isBusy;

        public bool IsBusy
        {
            get { return _isBusy; }
            set { SetProperty(ref _isBusy, value); }
        }

        private ObservableCollection<ArtistAlphabetInfo> _alphabetInfos;

        public ObservableCollection<ArtistAlphabetInfo> AlphabetInfos
        {
            get { return _alphabetInfos; }
            set { SetProperty(ref _alphabetInfos, value); }
        }

        private DataServiceCollection<Artist> _artists;

        public DataServiceCollection<Artist> Artists
        {
            get { return _artists; }
            set { SetProperty(ref _artists, value); }
        }

        private Artist _selectedArtist;

        public Artist SelectedArtist
        {
            get { return _selectedArtist; }
            set { SetProperty(ref _selectedArtist, value); }
        }

        private ArtistAlphabetInfo _selectedAlphabetInfo;


        private async void Load()
        {
            IsBusy = true;
            _cookie = await _chinookEntities.Login("andrew@chinookcorp.com", "1");
            //_cookie = await _chinookEntities.Login("steve@chinookcorp.com", "5");
            if (_cookie == null)
            {
                MessageDialog dialog = new MessageDialog("Login failed");
                dialog.ShowAsync();
            }
            else
            {
                AlphabetInfos = await _chinookEntities.ExecuteAsync<ArtistAlphabetInfo>(new Uri(BaseUrl + "ArtistSummary", UriKind.RelativeOrAbsolute), _cookie);    
            }
            
            IsBusy = false;
        }

        private async void LoadArtists(string letter)
        {
            IsBusy = true;
            var query = (DataServiceQuery<Artist>)_chinookEntities.Artists.Where(one => one.Name.ToUpper().Substring(0, 1) == letter.ToUpper());
            var data = await query.ExecuteAsync();

            Artists = new DataServiceCollection<Artist>(_chinookEntities);
            Artists.Load(data);

            IsBusy = false;
        }

        public SimpleCommand<ArtistAlphabetInfo> ArtistLetterSelectedCommand { get; set; }
        public void OnArtistLetterSelected(ArtistAlphabetInfo parameter)
        {
            _selectedAlphabetInfo = parameter;
            App.CurrentApp.RootFrame.Navigate(typeof(ArtistsPage), parameter);
            LoadArtists(parameter.Letter);
        }


        public SimpleCommand<object> AddArtistCommand { get; set; }
        public void OnAddArtist(object parameter)
        {
            var artist = new Artist { Name = _selectedAlphabetInfo.Letter + (new Random()).Next(10000, 100000).ToString() };
            Artists.Add(artist);
            SelectedArtist = artist;
        }

        public SimpleCommand<object> SaveArtistCommand { get; set; }
        public async void OnSaveArtists(object parameter)
        {
            IsBusy = true;
            try
            {
                await _chinookEntities.SaveAsync(App.CurrentApp.RootFrame.Dispatcher);
            }
            catch (Exception)
            {
                var dialog = new MessageDialog("Errors");
                dialog.ShowAsync();
            }

            IsBusy = false;
        }

        public SimpleCommand<object> DeleteArtistCommand { get; set; }
        public void OnDeleteArtist(object parameter)
        {
            if (SelectedArtist != null)
            {
                Artists.Remove(SelectedArtist);
                _selectedAlphabetInfo.Total -= 1;
                if (_selectedAlphabetInfo.Total == 0)
                {
                    AlphabetInfos.Remove(_selectedAlphabetInfo);
                }
            }
        }

        public SimpleCommand<object> EditArtistCommand { get; set; }
        public void OnEditeArtist(object parameter)
        {
            if (SelectedArtist != null)
            {
                App.CurrentApp.RootFrame.Navigate(typeof(EditArtistPage), parameter);
            }
        }



    }
}
